@@ -14,11 +14,13 @@ __title__ = "askitsu"
 __author__ = "ShomyKohai"
 __license__ = "MIT"
 __copyright__ = "Copyright 2022-present ShomyKohai"
-__version__ = "0.4.1"
+__version__ = "0.5.0"
 
 from .anime import *
 from .client import *
+from .character import *
 from .core import *
 from .error import *
 from .images import *
 from .manga import *
+from .users import *
