@@ -24,15 +24,17 @@ DEALINGS IN THE SOFTWARE.
 
 __all__ = ("Anime", "StreamLink", "Episode")
 
-import aiohttp
 from datetime import datetime
 from typing import List, Optional, Union
-from .core import Entry, BASE
+
+from .core import Entry
+from .http import HTTPClient
 
 
 class StreamLink:
     """
     Represent an :class:`Anime` stream link
+    (where you can watch the anime)
 
 
     Attributes
@@ -45,13 +47,16 @@ class StreamLink:
         Avaiable subs
     dub: :class:`list`
         Avaiable dub in streaming service
+    name: :class:`str`
+        Name of the stream service
     """
 
-    __slots__ = ("id", "url", "subs", "dub")
+    __slots__ = ("id", "url", "subs", "dub", "name")
 
-    def __init__(self, attributes: dict):
+    def __init__(self, attributes: dict, included: dict):
         data = attributes["attributes"]
         self.id: int = int(attributes["id"])
+        self.name: str = included["siteName"]
         self.url: str = data["url"]
         self.subs: list = data["subs"]
         self.dub: list = data["dubs"]
@@ -67,8 +72,6 @@ class Episode:
     -----------
     id: :class:`int`
         ID of the episode
-    created_at: Optional[:class:`datetime`]
-    updated_at: Optional[:class:`datetime`]
     synopsis: :class:`str`
         Synopsis of the episode
     description: :class:`str`
@@ -87,8 +90,6 @@ class Episode:
 
     __slots__ = (
         "id",
-        "created_at",
-        "updated_at",
         "synopsis",
         "description",
         "title",
@@ -96,31 +97,44 @@ class Episode:
         "number",
         "length",
         "thumbnail",
+        "_attributes",
     )
 
     def __init__(self, attributes: dict) -> None:
-        data = attributes["attributes"]
+        self._attributes = attributes["attributes"]
         self.id: int = int(attributes["id"])
-        self.created_at: Optional[datetime] = (
-            datetime.strptime(data["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["createdAt"])
-            else None
-        )
-        self.updated_at: Optional[datetime] = (
-            datetime.strptime(data["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["updatedAt"])
-            else None
-        )
-        self.synopsis: str = data["synopsis"]
-        self.description: str = data["description"]
-        self.title: str = data["canonicalTitle"]
-        self.season: int = data["seasonNumber"]
-        self.number: int = data["number"]
-        self.length: int = data["length"]
+        self.synopsis: str = self._attributes["synopsis"]
+        self.description: str = self._attributes["description"]
+        self.title: str = self._attributes["canonicalTitle"]
+        self.season: int = self._attributes["seasonNumber"]
+        self.number: int = self._attributes["number"]
+        self.length: int = self._attributes["length"]
         self.thumbnail: str = (
-            data["thumbnail"]["original"] if data["thumbnail"] else None
+            self._attributes["thumbnail"]["original"]
+            if self._attributes["thumbnail"]
+            else None
         )
 
+    @property
+    def created_at(self) -> Optional[datetime]:
+        """Date when this episode got added on Kitsu"""
+        try:
+            return datetime.strptime(
+                self._attributes["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def updated_at(self) -> Optional[datetime]:
+        """Last time when this episode got updated on Kitu"""
+        try:
+            return datetime.strptime(
+                self._attributes["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
 
 class Anime(Entry):
     """Represents an :class:`Anime` instance
@@ -130,26 +144,15 @@ class Anime(Entry):
     id: :class:`int`
         ID of the anime
     status: :class:`str`
-        Actual status of the given anime (Ex. "finished")
-    created_at: Optional[:class:`datetime`]
-    updated_at: Optional[:class:`datetime`]
-    started_at: Optional[:class:`datetime`]
-    ended_at: Optional[:class:`datetime`]
+        Actual status of the given anime (E.g. "finished")
     slug: :class:`str`
         String identifier. Work as id to fetch data
     synopsis: :class:`str`
         Description of the given anime
-    title: :class:`str`
-        Return canon title of the given anime
-
-        .. versionchanged:: 0.4.1
-
-        Now it returns an instance of :class:`askitsu.Title`
     canonical_title: :class:`str`
         Returns canonical title of the given anime
 
         .. versionadded:: 0.4.1
-
     episode_count: :class:`int`
         Episode number
     episode_lenght: :class:`int`
@@ -161,36 +164,10 @@ class Anime(Entry):
         Return True | False
     yt_id: :class:`str`
         Return id of the YouTube trailer
-    cover_image: :class:`CoverImage`
-        Return cover image dict with all sizes
-
-        .. versionchanged:: 0.4.1
-
-        Now it returns a cover image object
-
-    poster_image: :class:`PosterImage`
-        Return poster image dict with all sizes
-
-        .. versionchanged:: 0.4.1
-
-        Now it returns a poster image object
-
     rating_rank: :class:`int`
         Return rating rank (Position on the leaderboard based on rating)
     popularity_rank: :class:`int`
         Return popularity rank (Position on the leaderboard based on user preferences)
-    youtube_url: Optional[:class:`str`]
-        Return full url of YouTube trailer
-    url: :class:`str`
-        Returns url to Kitsu.io website
-
-        .. versionadded:: 0.4.0
-
-    stream_links: List[:class:`StreamLink`]
-        Return a list of :class:StreamLink
-
-        .. versionadded:: 0.4.0
-
     rating: :class:`float`
         The rating received from the community in a scale from 1 to 100
 
@@ -201,35 +178,16 @@ class Anime(Entry):
 
         .. versionadded:: 0.4.0
 
-    categories: List[:class:`Category`]
-        Categories of the anime
-
-        .. versionadded:: 0.4.0
-
     subtype: Literal['ONA', 'OVA', 'TV', 'movie', 'music', 'special']
         The subtype of the show
 
         .. versionadded:: 0.4.1
-
-    characters: Union[:class:`Character`, List[:class:`Character`]
-        Get all characters (Max 20)
-
-        .. versionadded:: 0.4.1
-
-        Note
-        --------------
-        Use :meth:`askitsu.Client.get_characters` if you want to set a limit\n
-        The limit with this property is automatically set to 20 (The highest)
     """
 
     __slots__ = (
         "id",
         "entry_type",
         "status",
-        "created_at",
-        "updated_at",
-        "started_at",
-        "ended_at",
         "slug",
         "synopsis",
         "canonical_title",
@@ -243,53 +201,59 @@ class Anime(Entry):
         "rating",
         "age_rating",
         "subtype",
-        "_session",
+        "_http",
         "_titles",
-        "_attributes"
+        "_attributes",
     )
 
-    def __init__(self, attributes: dict, session: aiohttp.ClientSession, *args) -> None:
+    def __init__(self, attributes: dict, http: HTTPClient, *args) -> None:
         data = attributes["attributes"]
-        self._session = session
+        self._http = http
         self.entry_type = "anime"
         self.episode_count: int = data["episodeCount"]
         self.episode_length: int = data["episodeLength"]
         self.total_length: int = data["totalLength"]
         self.nsfw: bool = data["nsfw"]
         self.yt_id: str = data["youtubeVideoId"]
-        super().__init__(attributes["id"], self.entry_type, data, session, *args)
+        super().__init__(attributes["id"], self.entry_type, data, http, *args)
+
+    def __repr__(self) -> str:
+        return f"<Anime name='{self.canonical_title}' id={self.id}>"
 
     @property
     def youtube_url(self) -> Optional[str]:
+        """Return full url of YouTube trailer"""
         return f"https://www.youtube.com/watch?v={self.yt_id}" if self.yt_id else None
 
-    async def _fetch_stream_links(self) -> Optional[List[StreamLink]]:
-        async with self._session.get(
-            url=f"{BASE}/anime/{self.id}/streaming-links"
-        ) as data:
-            fetched_data = await data.json()
-            return (
-                [StreamLink(links) for links in fetched_data["data"]]
-                if fetched_data
-                else None
-            )
-
     @property
     async def stream_links(self) -> Optional[List[StreamLink]]:
-        return await self._fetch_stream_links()
+        """
+        Return a list of :class:`StreamLink`
+
+        .. versionadded:: 0.4.0
+        """
+        data = await self._http.get_data(
+            url=f"anime/{self.id}/streaming-links?include=streamer"
+        )
+        try:
+            return [
+                StreamLink(links, included["attributes"])
+                for links, included in zip(data["data"], data["included"])
+            ]
+        except KeyError:
+            return []
 
     async def episodes(self, limit: int = 12) -> Union[Episode, List[Episode]]:
         """
-        Returns a a episode or a list of episodes
+        Returns an episode or a list of episodes
 
         .. versionadded:: 0.4.0
 
         limit: :class:`int`
             Limit of episodes to fetch. Defaults to 12 (Max 25).
         """
-        async with self._session.get(
-            url=f"{BASE}/anime/{self.id}/episodes?page[limit]={limit}"
-        ) as data:
-            fetched_data = await data.json()
-            episodes = [Episode(attributes) for attributes in fetched_data["data"]]
-            return episodes if len(episodes) > 1 else episodes[0]
+        data = await self._http.get_data(
+            url=f"anime/{self.id}/episodes?page[limit]={limit}"
+        )
+        episodes = [Episode(attributes) for attributes in data["data"]]
+        return episodes if len(episodes) > 1 else episodes[0]
