@@ -23,26 +23,15 @@ DEALINGS IN THE SOFTWARE.
 """
 
 import aiohttp
-from colorama import Fore, Style
-from typing import (
-    Any, 
-    List,
-    Literal,
-    Optional,
-    overload,
-    Union
-)
+from colorama import Fore, Style  # type: ignore
+from typing import Dict, List, Literal, Optional, overload, Type, Union
 from .anime import Anime, StreamLink
 from .character import Character
-from .core import Review, BASE
-from .error import (
-    BadApiRequest, 
-    InvalidArgument,
-    NotAuthenticated
-)
-from . import __version__
+from .core import Object, Review
+from .error import InvalidArgument
+from .http import HTTPClient
 from .manga import Manga
-
+from .users import User
 
 
 class Client:
@@ -52,83 +41,70 @@ class Client:
     -----------
     token: :class:`str`
         Access token to make authenticated requests.\n
-        Useful when you need to fetch NSFW content, interact with users or post something. 
+        Useful when you need to fetch NSFW content, interact with users or post something.
 
         .. versionadded:: 0.4.1
 
     session: Optional[:class:`aiohttp.ClientSession`]
         An object that represents the effective connection
-    
+
     Attributes
     -----------
     token: :class:`str`
         Token passed to the session.
     """
 
-    def __init__(self, token: str = None, *, session: Optional[aiohttp.ClientSession] = None) -> None:
-        self._entries = {
+    def __init__(
+        self, token: str = None, *, session: Optional[aiohttp.ClientSession] = None
+    ) -> None:
+        self._entries: Dict[str, Union[Type[Anime], Type[Manga], Type[Character]]] = {
             "anime": Anime,
             "manga": Manga,
-            "characters": Character
+            "characters": Character,
         }
-        self.__authorization = f"Bearer {token}" if token else ""
-        self._session: aiohttp.ClientSession = session or aiohttp.ClientSession()
-        self.__headers = {
-            "Accept": "application/vnd.api+json",
-            "Content-Type": "application/vnd.api+json",
-            "User-Agent": f"askitsu (https://github.com/ShomyKohai/askitsu {__version__})",
-            "Authorization": self.__authorization
-        }
-        self.token = token
+        self.http: HTTPClient = HTTPClient(
+            session=session or aiohttp.ClientSession(), token=token
+        )
 
-    async def _get_data(self, url: str) -> Any:
-        async with self._session.get(url=url, headers=self.__headers) as response:
-            response_data = await response.json()
-            if response.status == 200:
-                return response_data
-            if response.status == 404:
-                return None
-            if response.status == 401:
-                raise NotAuthenticated
-            if response.status == 400:
-                raise BadApiRequest(response_data["errors"][0])
+    @property
+    def session(self) -> aiohttp.ClientSession:
+        return self.http.session
+
+    @property
+    def token(self) -> Optional[str]:
+        return self.http.token
 
     @overload
-    async def search(
-       self, type: Literal["anime"], query: str
-    ) -> Optional[Anime]:
+    async def search(self, type: Literal["anime"], query: str) -> Optional[Anime]:
         ...
-    
+
     @overload
     async def search(
-       self, type: Literal["anime"], query: str, limit: int = ...
+        self, type: Literal["anime"], query: str, limit: int
     ) -> Optional[List[Anime]]:
         ...
 
     @overload
-    async def search(
-       self, type: Literal["manga"], query: str
-    ) -> Optional[Manga]:
+    async def search(self, type: Literal["manga"], query: str) -> Optional[Manga]:
         ...
-    
+
     @overload
     async def search(
-       self, type: Literal["manga"], query: str, limit: int = ...
+        self, type: Literal["manga"], query: str, limit: int
     ) -> Optional[List[Manga]]:
         ...
-    
+
     @overload
     async def search(
-       self, type: Literal["characters"], query: str
+        self, type: Literal["characters"], query: str
     ) -> Optional[Character]:
         ...
-    
+
     @overload
     async def search(
-       self, type: Literal["characters"], query: str, limit: int = ...
+        self, type: Literal["characters"], query: str, limit: int
     ) -> Optional[List[Character]]:
-        ...    
-       
+        ...
 
     async def search(
         self, type: Literal["anime", "manga", "characters"], query: str, limit: int = 1
@@ -150,32 +126,31 @@ class Client:
 
         """
         type_lower = type.lower()
-        entry = self._entries.get(type_lower)
+        try:
+            entry = self._entries[type_lower]
+        except (KeyError, TypeError):
+            raise InvalidArgument
         filter = "name" if type_lower == "characters" else "text"
-        fetched_data = await self._get_data(
-            f"{BASE}/{type_lower}?filter%5B{filter}%5D={query}&page%5Blimit%5D={limit}"
+        fetched_data = await self.http.get_data(
+            f"{type_lower}?filter%5B{filter}%5D={query}&page%5Blimit%5D={limit}"
         )
         if not fetched_data["data"]:
             return None
         if len(fetched_data["data"]) == 1:
-            return entry(fetched_data["data"][0], self._session)
+            return entry(fetched_data["data"][0], http=self.http)
         return [
-            entry(attributes=attributes, session=self._session)
+            entry(attributes=attributes, http=self.http)
             for attributes in fetched_data["data"]
-        ]
+        ]  # type: ignore
 
     @overload
-    async def search_anime(
-        self, query: str
-    ) -> Optional[Anime]:
+    async def search_anime(self, query: str) -> Optional[Anime]:
         ...
 
     @overload
-    async def search_anime(
-        self, query: str, limit: int = ...
-    ) -> Optional[List[Anime]]:
+    async def search_anime(self, query: str, limit: int) -> Optional[List[Anime]]:
         ...
-   
+
     async def search_anime(
         self, query: str, limit: int = 1
     ) -> Optional[Union[Anime, List[Anime]]]:
@@ -193,15 +168,12 @@ class Client:
         return await self.search("anime", query=query, limit=limit)
 
     @overload
-    async def search_manga(
-        self, query: str
-    ) -> Optional[Manga]:
+    async def search_manga(self, query: str) -> Optional[Manga]:
         ...
+
     @overload
-    async def search_manga(
-        self, query: str, limit: int = ...
-    ) -> Optional[List[Manga]]:
-        ...    
+    async def search_manga(self, query: str, limit: int) -> Optional[List[Manga]]:
+        ...
 
     async def search_manga(
         self, query: str, limit: int = 1
@@ -220,14 +192,12 @@ class Client:
         return await self.search("manga", query=query, limit=limit)
 
     @overload
-    async def search_character(
-        self, query: str
-    ) -> Optional[Character]:
+    async def search_character(self, query: str) -> Optional[Character]:
         ...
 
     @overload
     async def search_character(
-        self, query: str, limit: int = ...
+        self, query: str, limit: int
     ) -> Optional[List[Character]]:
         ...
 
@@ -253,28 +223,31 @@ class Client:
         """
         return await self.search("characters", query=query, limit=limit)
 
+    async def search_user(self, name: str) -> Optional[User]:
+        """
+        Fetch a user by their username
+
+        name: :class:`str`
+            Nickname of the user to fetch
+        """
+        data = await self.http.get_data(url=f"users?filter%5Bslug%5D={name}")
+        return User(data["data"][0], http=self.http) if data["data"] else None
 
     @overload
-    async def get_entry(
-        self, type: Literal["anime"], id: int
-    ) -> Anime:
+    async def get_entry(self, type: Literal["anime"], id: int) -> Anime:
         ...
 
     @overload
-    async def get_entry(
-        self, type: Literal["manga"], id: int
-    ) -> Manga:
-        ... 
+    async def get_entry(self, type: Literal["manga"], id: int) -> Manga:
+        ...
 
     @overload
-    async def get_entry(
-        self, type: Literal["characters"], id: int
-    ) -> Character:
+    async def get_entry(self, type: Literal["characters"], id: int) -> Character:
         ...
 
     async def get_entry(
         self, type: Literal["anime", "manga", "characters"], id: int
-    ) -> Union[Anime, Manga, Character]:
+    ) -> Optional[Union[Anime, Manga, Character]]:
         """|coro|
 
         Get an entry object (`Anime` | `Manga` | `Character`) by an id
@@ -287,10 +260,13 @@ class Client:
             ID of the media
         """
         type_lower = type.lower()
-        entry = self._entries.get(type_lower)
-        fetched_data = await self._get_data(f"{BASE}/{type_lower}/{id}")
+        try:
+            entry = self._entries[type_lower]
+        except (KeyError, TypeError):
+            raise InvalidArgument
+        fetched_data = await self.http.get_data(f"{type_lower}/{id}")
         return (
-            entry(attributes=fetched_data["data"], session=self._session)
+            entry(attributes=fetched_data["data"], http=self.http)
             if fetched_data
             else None
         )
@@ -321,7 +297,7 @@ class Client:
         """
         return await self.get_entry("manga", id=id)
 
-    async def get_stream_links(self, anime: Anime) -> List[StreamLink]:
+    async def get_stream_links(self, anime: Anime) -> Optional[List[StreamLink]]:
         """|coro|
 
         Return all streaming link of an Anime object
@@ -331,26 +307,31 @@ class Client:
         anime: :class:`Anime`
             The anime to get stream links
         """
-        if not isinstance(anime, Anime):
+        if not isinstance(anime, Anime) and not isinstance(anime, Object):
             raise InvalidArgument(
                 f"{Fore.RED}'{anime}' is not an istance of Anime\n"
                 f"Make sure you pass a valid argument to {Fore.LIGHTCYAN_EX}get_stream_links{Style.RESET_ALL}"
             )
-        fetched_data = await self._get_data(
-            url=f"{BASE}/anime/{anime.id}/streaming-links"
+        fetched_data = await self.http.get_data(
+            url=f"anime/{anime.id}/streaming-links?include=streamer"
         )
-        return [StreamLink(links) for links in fetched_data["data"]]
-
+        try:
+            return [
+                StreamLink(links, included["attributes"])
+                for links, included in zip(
+                    fetched_data["data"], fetched_data["included"]
+                )
+            ]
+        except KeyError:
+            return None
 
     @overload
-    async def get_characters(
-        self, entry: Union[Anime, Manga]
-    ) -> Character:
+    async def get_characters(self, entry: Union[Anime, Manga]) -> Character:
         ...
 
     @overload
     async def get_characters(
-        self, entry: Union[Anime, Manga], limit: int = ...
+        self, entry: Union[Anime, Manga], limit: int
     ) -> List[Character]:
         ...
 
@@ -368,28 +349,24 @@ class Client:
         limit: :class:`int`
             Number of characters' fetch
         """
-        fetched_data = await self._get_data(
-            url=f"{BASE}/{entry.entry_type}/{entry.id}/characters?include=character&page%5Blimit%5D={limit}"
+        fetched_data = await self.http.get_data(
+            url=f"{entry.entry_type}/{entry.id}/characters?include=character&page%5Blimit%5D={limit}"
         )
         characters_roles = [link["attributes"]["role"] for link in fetched_data["data"]]
         characters = [
-            Character(attributes, role=role, entry_id=entry.id) 
+            Character(attributes, role=role, entry_id=entry.id)
             for attributes, role in zip(fetched_data["included"], characters_roles)
         ]
         return characters if len(characters) > 1 else characters[0]
 
     @overload
-    async def get_trending_entry(
-        self, type: Literal["anime"]
-    ) -> Optional[List[Anime]]:
+    async def get_trending_entry(self, type: Literal["anime"]) -> Optional[List[Anime]]:
         ...
 
     @overload
-    async def get_trending_entry(
-        self, type: Literal["manga"]
-    ) -> Optional[List[Manga]]:
+    async def get_trending_entry(self, type: Literal["manga"]) -> Optional[List[Manga]]:
         ...
-    
+
     async def get_trending_entry(
         self, type: Literal["anime", "manga"]
     ) -> Union[List[Anime], List[Manga], None]:
@@ -405,29 +382,28 @@ class Client:
             Entry to fetch its trending
         """
         type_lower = type.lower()
-        if type_lower == "characters":
+        if type_lower not in ("anime", "manga"):
             raise InvalidArgument(
-                f"{Fore.RED}Characters can't be fetched in trending list\n"
+                f"{Fore.RED}{type.capitalize} can't be fetched in trending list\n"
                 f"Please pass 'anime' or 'manga' as parameter to {Fore.LIGHTCYAN_EX}get_trending_entry{Style.RESET_ALL}"
             )
-        entry = self._entries.get(type_lower)
-        fetched_data = await self._get_data(f"{BASE}/trending/{type_lower}")
+        try:
+            entry = self._entries[type_lower]
+        except (KeyError, TypeError):
+            raise InvalidArgument
+        fetched_data = await self.http.get_data(f"trending/{type_lower}")
         return [
-            entry(attributes=attributes, session=self._session)
+            entry(attributes=attributes, http=self.http)
             for attributes in fetched_data["data"]
-        ]
-
+        ]  # type: ignore
 
     @overload
-    async def get_reviews(
-        self, 
-        entry: Union[Manga, Anime]
-    ) -> Optional[Review]:
+    async def get_reviews(self, entry: Union[Manga, Anime]) -> Optional[Review]:
         ...
 
     @overload
     async def get_reviews(
-        self, entry: Union[Manga, Anime], limit: int = ...
+        self, entry: Union[Manga, Anime], limit: int
     ) -> Optional[List[Review]]:
         ...
 
@@ -447,8 +423,8 @@ class Client:
         limit: :class:`int`
             Limit to reviews to fetch
         """
-        fetched_data = await self._get_data(
-            url=f"{BASE}/{entry.entry_type}/{entry.id}/reviews?page%5Blimit%5D={limit}"
+        fetched_data = await self.http.get_data(
+            url=f"{entry.entry_type}/{entry.id}/reviews?page%5Blimit%5D={limit}"
         )
         reviews = [
             Review(entry.id, entry.entry_type, reviews)
@@ -456,8 +432,38 @@ class Client:
         ]
         return (reviews if limit > 1 else reviews[0]) if reviews else None
 
+    async def get_user(self, id: int) -> Optional[User]:
+        """|coro|
+
+        Get a user by their id
+
+        .. versionadded:: 0.5.0
+
+        Parameters
+        -----------
+        id: :class:`int`
+            The id of the user to fetch
+        """
+        data = await self.http.get_data(url=f"users/{id}")
+        return User(data["data"], http=self.http) if data["data"] else None
+
+    async def check_user(self, slug: str) -> bool:
+        """|coro|
+
+        Check if the user exists on Kitsu
+
+        .. versionadded:: 0.5.0
+
+        Parameters
+        -----------
+        slug: :class:`str`
+            Nickname of the user
+        """
+        data = await self.http.get_data(url=f"users?filter%5Bslug%5D={slug}")
+        return bool(data["data"])
+
     async def close(self) -> None:
         """
         Close client connection
         """
-        return await self._session.close()
+        return await self.http.close()
