@@ -24,13 +24,15 @@ DEALINGS IN THE SOFTWARE.
 
 from __future__ import annotations
 
-import aiohttp
 from datetime import datetime
 from typing import List, Literal, Optional, Union
+
 from .character import Character
+from .http import HTTPClient
 from .images import CoverImage, PosterImage
 
-BASE: str = "https://kitsu.io/api/edge/"
+
+__all__ = ("Category", "Review", "Title", "Object")
 
 
 class Entry:
@@ -39,10 +41,6 @@ class Entry:
         "id",
         "entry_type",
         "status",
-        "created_at",
-        "updated_at",
-        "started_at",
-        "ended_at",
         "slug",
         "synopsis",
         "canonical_title",
@@ -50,31 +48,19 @@ class Entry:
         "popularity_rank",
         "rating",
         "age_rating",
-        "subtype"
+        "subtype",
+        "_titles",
+        "_attributes",
+        "_http",
     )
 
-    def __init__(self, _id: int, _type: str, attributes: dict, session: aiohttp.ClientSession):
-        self._session = session
+    def __init__(self, _id: int, _type: str, attributes: dict, http: HTTPClient):
+        self._http = http
         self._attributes = attributes
         self._titles: dict = attributes["titles"]
         self.id = int(_id)
         self.entry_type = _type
         self.status: str = attributes["status"]
-        self.created_at: Optional[datetime] = (
-            datetime.strptime(attributes["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (attributes["createdAt"])
-            else None
-        )
-        self.updated_at: Optional[datetime] = (
-            datetime.strptime(attributes["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (attributes["updatedAt"])
-            else None
-        )
-        self.ended_at: Optional[datetime] = (
-            datetime.strptime(attributes["endDate"], "%Y-%m-%d")
-            if (attributes["endDate"])
-            else None
-        )
         self.slug: str = attributes["slug"]
         self.synopsis: str = attributes["synopsis"]
         self.canonical_title: str = attributes["canonicalTitle"]
@@ -84,63 +70,131 @@ class Entry:
         self.age_rating: Literal["G", "PG", "R", "R18"] = attributes["ageRating"]
         self.subtype: str = attributes["subtype"]
 
+    @property
+    def created_at(self) -> Optional[datetime]:
+        """When the media got added in Kitsu database"""
+        try:
+            return datetime.strptime(
+                self._attributes["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def updated_at(self) -> Optional[datetime]:
+        """Last time the media got updated in Kitsu database"""
+        try:
+            return datetime.strptime(
+                self._attributes["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def started_at(self) -> Optional[datetime]:
+        """Date when the Media started"""
+        try:
+            return datetime.strptime(self._attributes["startDate"], "%Y-%m-%d")
+        except ValueError:
+            return None
+
+    @property
+    def ended_at(self) -> Optional[datetime]:
+        """Date when Media ended"""
+        try:
+            return datetime.strptime(self._attributes["endDate"], "%Y-%m-%d")
+        except ValueError:
+            return None
 
     @property
     def url(self) -> str:
+        """
+        url: :class:`str`
+        Returns url to Kitsu.io website
+
+        .. versionadded:: 0.4.0
+        """
         return f"https://kitsu.io/{self.entry_type}/{self.slug}"
 
     @property
     def title(self) -> Title:
+        """
+        Return canon title of the given Media
+
+        .. versionchanged:: 0.4.1
+
+        Now it returns an instance of :class:`askitsu.Title`
+        """
         return Title(self._titles, self.id, self.entry_type)
-    
+
     @property
     def poster_image(self) -> PosterImage:
-        return PosterImage(
-            self._attributes["posterImage"],
-            self.id,
-            self.entry_type
-        )
+        """
+        Return poster image dict with all sizes
+
+        .. versionchanged:: 0.4.1
+
+        Now it returns a poster image object
+        """
+        return PosterImage(self._attributes["posterImage"], self.id, self.entry_type)
 
     @property
     def cover_image(self) -> CoverImage:
-        return CoverImage(
-            self._attributes["coverImage"],
-            self.id,
-            self.entry_type
-        )
+        """
+        Return cover image dict with all sizes
+
+        .. versionchanged:: 0.4.1
+
+        Now it returns a cover image object
+        """
+        return CoverImage(self._attributes["coverImage"], self.id, self.entry_type)
 
     @property
     async def categories(self) -> List[Category]:
-        async with self._session.get(
-            url=f"{BASE}/{self.entry_type}/{self.id}/categories"
-        ) as data:
-            fetched_data = await data.json()
-            categories = [Category(attributes) for attributes in fetched_data["data"]]
-            return categories
+        """
+        Categories of the Media
+
+        .. versionadded:: 0.4.0
+        """
+        data = await self._http.get_data(url=f"{self.entry_type}/{self.id}/categories")
+        return [Category(attributes) for attributes in data["data"]]
 
     @property
     async def characters(self) -> Union[Character, List[Character]]:
-        async with self._session.get(
-            url=f"{BASE}/{self.entry_type}/{self.id}/characters?include=character&page%5Blimit%5D=20"
-        ) as data:
-            fetched_data = await data.json()
-            characters_roles = [link["attributes"]["role"] for link in fetched_data["data"]]
-            characters = [
-                Character(attributes, role=role, entry_id=self.id)
-                for attributes, role in zip(fetched_data["included"], characters_roles)
-            ]
-            return characters if len(characters) > 1 else characters[0]
+        """
+        Get all characters (Max 20)
+
+        .. versionadded:: 0.4.1
+
+        Note
+        --------------
+        Use :meth:`askitsu.Client.get_characters` if you want to set a limit\n
+        The limit with this property is automatically set to 20 (The highest)
+        """
+        data = await self._http.get_data(
+            url=f"{self.entry_type}/{self.id}/characters?include=character&page%5Blimit%5D=20"
+        )
+        characters_roles = [link["attributes"]["role"] for link in data["data"]]
+        characters = [
+            Character(attributes, role=role, entry_id=self.id)
+            for attributes, role in zip(data["included"], characters_roles)
+        ]
+        return characters if len(characters) > 1 else characters[0]
 
     async def reviews(self, limit: int = 1) -> Optional[Union[Review, List[Review]]]:
-        async with self._session.get(
-            url=f"{BASE}/{self.entry_type}/{self.id}/reviews?page%5Blimit%5D={limit}"
-        ) as data:
-            fetched_data = await data.json()
-            reviews = [
-                Review(self.id, self.entry_type, reviews)
-                for reviews in fetched_data["data"]
-            ]
-            return (reviews if limit > 1 else reviews[0]) if reviews else None
+        """
+        Get all the reviews of the Media
+
+        .. versionadded:: 0.3.0
+        """
+        data = await self._http.get_data(
+            url=f"{self.entry_type}/{self.id}/reviews?page%5Blimit%5D={limit}"
+        )
+        reviews = [
+            Review(self.id, self.entry_type, reviews) for reviews in data["data"]
+        ]
+        return (reviews if limit > 1 else reviews[0]) if reviews else None
+
 
 class Category:
     """
@@ -150,8 +204,6 @@ class Category:
 
     Attributes
     ------------
-    created_at: :class:`datetime`
-    updated_at: :class:`datetime`
     title: :class:`str`
         Title of the category
     description: :class:`str`
@@ -162,24 +214,37 @@ class Category:
         If the category is NSFW or not
     """
 
-    __slots__ = ("created_at", "updated_at", "title", "description", "slug", "nsfw")
+    __slots__ = ("title", "description", "slug", "nsfw", "_attributes")
 
     def __init__(self, attributes: dict) -> None:
-        data = attributes["attributes"]
-        self.created_at: Optional[datetime] = (
-            datetime.strptime(data["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["createdAt"])
-            else None
-        )
-        self.updated_at: Optional[datetime] = (
-            datetime.strptime(data["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["updatedAt"])
-            else None
-        )
-        self.title: str = data["title"]
-        self.description: str = data["description"]
-        self.slug: str = data["slug"]
-        self.nsfw: bool = data["nsfw"]
+        self._attributes = attributes["attributes"]
+        self.title: str = self._attributes["title"]
+        self.description: str = self._attributes["description"]
+        self.slug: str = self._attributes["slug"]
+        self.nsfw: bool = self._attributes["nsfw"]
+
+    def __repr__(self) -> str:
+        return f"<Category title={self.title}>"
+
+    @property
+    def created_at(self) -> Optional[datetime]:
+        """When a category got added in Kitu DB"""
+        try:
+            return datetime.strptime(
+                self._attributes["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def updated_at(self) -> Optional[datetime]:
+        """Last time a category got updated"""
+        try:
+            return datetime.strptime(
+                self._attributes["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
 
 
 class Review:
@@ -253,7 +318,10 @@ class Title:
     ja_jp: :class:`str`
         The title of the media in Japanese
     """
-    def __init__(self, data: dict, entry_id: int = None, entry_type: str = None) -> None:
+
+    def __init__(
+        self, data: dict, entry_id: int = None, entry_type: str = None
+    ) -> None:
         self.__data = data
         self.entry_id = entry_id
         self.entry_type = entry_type
@@ -272,4 +340,31 @@ class Title:
 
     @property
     def ja_jp(self) -> Optional[str]:
-        return self.__data.get("ja_jp")
\ No newline at end of file
+        return self.__data.get("ja_jp")
+
+
+class Object:
+    """
+    Represent a generic Object.
+    This can be useful if you want to use some methods that require a
+    specific istance.
+
+    Example:
+    If you want to fetch the characters using :meth:`askitsu.Client.get_characters`
+    without a :class:`Manga` or :class:`Anime` istance, you can use this class by giving
+    an id and an entry_type
+
+    Attributes
+    --------------
+    id: :class:`int`
+        The id of the object
+    entry_type: Optional[:class:`str`]
+        The type of the object
+    """
+
+    def __init__(self, id: int, *, type: str = None) -> None:
+        self.id: int = id
+        self.entry_type: Optional[str] = type
+
+    def __repr__(self) -> str:
+        return f"<Object id={self.id} entry_type={self.entry_type}>"
