@@ -21,7 +21,7 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
-from colorama import Fore, Style
+from colorama import Fore, Style  # type: ignore
 
 
 class AskitsuException(Exception):
@@ -61,7 +61,7 @@ class InvalidArgument(AttributeError):
         Error message to pass
     """
 
-    def __init__(self, msg) -> None:
+    def __init__(self, msg: str = None) -> None:
         super().__init__(msg)
 
 
@@ -78,7 +78,7 @@ class NotAuthenticated(HTTPError):
             f"{Fore.RED}You are not authenticated.\n"
             f"Check if the called API request need authentication "
             f"or if you passed valid credentials{Style.RESET_ALL}",
-            401
+            401,
         )
 
 
@@ -91,7 +91,18 @@ class BadApiRequest(HTTPError):
 
     def __init__(self, response: str) -> None:
         super().__init__(
-            f"{Fore.RED}An error occured.\n"
+            f"{Fore.RED}An error occured.\n"  # type: ignore
             f"{response['detail']} - Response code: {400}{Style.RESET_ALL}",
-            400
+            400,
         )
+
+
+class NotFound(HTTPError):
+    """
+    Raises when a 404 error code takes place
+
+    .. versionadded:: 0.5.0
+    """
+
+    def __init__(self) -> None:
+        super().__init__(f"{Fore.RED}Resource not found.\n", 404)
