@@ -24,10 +24,11 @@ DEALINGS IN THE SOFTWARE.
 
 __all__ = ("Manga", "Chapter")
 
-import aiohttp
 from datetime import datetime
 from typing import Union, List, Optional
-from .core import Entry, BASE
+
+from .core import Entry
+from .http import HTTPClient
 
 
 class Chapter:
@@ -39,10 +40,6 @@ class Chapter:
     -----------
     id: :class:`int`
         ID of the chapter
-    created_at: Optional[:class:`datetime`]
-    updated_at: Optional[:class:`datetime`]
-    published: Optional[:class:`datetime`]
-        When the chapter got published (YYYY-mm-dd)
     synopsis: :class:`str`
         Synopsis of the chapter
     description: :class:`str`
@@ -61,36 +58,20 @@ class Chapter:
 
     __slots__ = (
         "id",
-        "created_at",
-        "updated_at",
         "synopsis",
         "description",
-        "published",
         "title",
         "volume_number",
         "chapter",
         "length",
         "thumbnail",
+        "_attributes",
     )
 
     def __init__(self, attributes: dict) -> None:
         data = attributes["attributes"]
+        self._attributes = attributes
         self.id: int = int(attributes["id"])
-        self.created_at: Optional[datetime] = (
-            datetime.strptime(data["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["createdAt"])
-            else None
-        )
-        self.updated_at: Optional[datetime] = (
-            datetime.strptime(data["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ")
-            if (data["updatedAt"])
-            else None
-        )
-        self.published: Optional[datetime] = (
-            datetime.strptime(data["published"], "%Y-%m-%d")
-            if (data["published"])
-            else None
-        )
         self.synopsis: str = data["synopsis"]
         self.description: str = data["description"]
         self.title: str = data["canonicalTitle"]
@@ -101,6 +82,33 @@ class Chapter:
             data["thumbnail"]["original"] if data["thumbnail"] else None
         )
 
+    @property
+    def created_at(self) -> Optional[datetime]:
+        """Date when a chapter got added on Kitsu DB"""
+        try:
+            return datetime.strptime(
+                self._attributes["createdAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def updated_at(self) -> Optional[datetime]:
+        try:
+            return datetime.strptime(
+                self._attributes["updatedAt"], "%Y-%m-%dT%H:%M:%S.%fZ"
+            )
+        except ValueError:
+            return None
+
+    @property
+    def published(self) -> Optional[datetime]:
+        """Date when a Chapter got published (YYYY-mm-dd)"""
+        try:
+            return datetime.strptime(self._attributes["published"], "%Y-%m-%d")
+        except ValueError:
+            return None
+
 
 class Manga(Entry):
     """Represents a :class:`Manga` instance
@@ -110,11 +118,7 @@ class Manga(Entry):
     id: :class:`int`
         ID of the manga
     status: :class:`str`
-        Actual status of the given manga (Ex. "finished")
-    created_at: Optional[:class:`datetime`]
-    updated_at: Optional[:class:`datetime`]
-    started_at: Optional[:class:`datetime`]
-    ended_at: Optional[:class:`datetime`]
+        Actual status of the given manga (E.g. "finished")
     slug: :class:`str`
         String identifier. Work as id to fetch data
     synopsis: :class:`str`
@@ -129,36 +133,17 @@ class Manga(Entry):
         Returns canonical title of the given manga
 
         .. versionadded:: 0.4.1
-        
+
     chapter_count: :class:`int`
         Number of chapters
     volume_count: :class:`int`
         Number of volumes
     serialization: :class:`str`
         Return manga serialization
-    cover_image: :class:`CoverImage`
-        Return cover image dict with all sizes
-
-        .. versionchanged:: 0.4.1
-
-        Now it returns a cover image object
-
-    poster_image: :class:`PosterImage`
-        Return poster image dict with all sizes
-
-        .. versionchanged:: 0.4.1
-
-        Now it returns a poster image object
-
     rating_rank: :class:`int`
         Return rating rank
     popularity_rank: :class:`int`
         Return popularity rank position
-    url: :class:`str`
-        Returns url to Kitsu.io website
-
-        .. versionadded:: 0.4.0
-
     rating: :class:`float`
         The rating received from the community in a scale from 1 to 100
 
@@ -169,35 +154,16 @@ class Manga(Entry):
 
         .. versionadded:: 0.4.0
 
-    categories: List[:class:`Category`]
-        Categories of the manga
-
-        .. versionadded:: 0.4.0
-
     subtype: Literal['doujin', 'manga', 'manhua', 'manhwa', 'novel', 'oel', 'oneshot']
         The subtype of the manga
 
         .. versionadded:: 0.4.1
-
-    characters: Union[:class:`Character`, List[:class:`Character`]
-        Get all characters (Max 20)
-
-        .. versionadded:: 0.4.1
-
-        Note
-        --------------
-        Use :meth:`askitsu.Client.get_characters` if you want to set a limit\n
-        The limit with this property is automatically set to 20 (The highest)
     """
 
     __slots__ = (
         "id",
         "entry_type",
         "status",
-        "created_at",
-        "updated_at",
-        "started_at",
-        "ended_at",
         "slug",
         "synopsis",
         "canonical_title",
@@ -209,19 +175,22 @@ class Manga(Entry):
         "rating",
         "age_rating",
         "subtype",
-        "_session",
+        "_http",
         "_titles",
-        "_attributes"
+        "_attributes",
     )
 
-    def __init__(self, attributes: dict, session: aiohttp.ClientSession, *args) -> None:
+    def __init__(self, attributes: dict, http: HTTPClient, *args) -> None:
         data = attributes["attributes"]
-        self._session = session
+        self._http = http
         self.entry_type: str = "manga"
         self.chapter_count: int = data["chapterCount"]
         self.volume_count: int = data["volumeCount"]
         self.serialization: str = data["serialization"]
-        super().__init__(attributes["id"], self.entry_type, data, session, *args)
+        super().__init__(attributes["id"], self.entry_type, data, http, *args)
+
+    def __repr__(self) -> str:
+        return f"<Manga name='{self.canonical_title}' id={self.id}>"
 
     async def chapters(self, limit: int = 12) -> Union[Chapter, List[Chapter]]:
         """
@@ -232,9 +201,8 @@ class Manga(Entry):
         limit: :class:`int`
             Limit of chapters to fetch. Defaults to 12 (Max 20).
         """
-        async with self._session.get(
-            url=f"{BASE}/manga/{self.id}/chapters?page[limit]={limit}"
-        ) as data:
-            fetched_data = await data.json()
-            chapters = [Chapter(attributes) for attributes in fetched_data["data"]]
-            return chapters if len(chapters) > 1 else chapters[0]
+        data = await self._http.get_data(
+            url=f"manga/{self.id}/chapters?page[limit]={limit}"
+        )
+        chapters = [Chapter(attributes) for attributes in data["data"]]
+        return chapters if len(chapters) > 1 else chapters[0]
